@@ -22,7 +22,7 @@ from zlib import crc32
 
 from samsa import handlers
 from samsa.exceptions import (ERROR_CODES, InvalidVersionError,
-    SocketDisconnectedError)
+    SocketDisconnectedError, MessageTooLargeError)
 from samsa.utils import attribute_repr
 from samsa.utils.functional import methodimap
 from samsa.utils.namedstruct import NamedStruct
@@ -85,7 +85,8 @@ class VersionHeaderMap(dict):
 
 
 class Message(object):
-    __slots__ = ('_headers', 'raw', 'offset')
+    __slots__ = ('_headers', '_payload', '_raw',
+                 '_offset', '_len', '_valid')
 
     Header = NamedStruct('Header', (
         ('i', 'length'),
@@ -103,22 +104,31 @@ class Message(object):
     })
 
     def __init__(self, raw, offset=0):
-        self.raw = raw
-        self.offset = offset
-
+        # Process headers
         self._headers = []
-        header = self.Header.unpack_from(self.raw)
+        header = self.Header.unpack_from(raw)
         self._headers.append(header)
-
         versioned_header = self.VersionHeaders[header.magic].unpack_from(
-            self.raw, offset=self.Header.size
+            raw, offset=self.Header.size
         )
         self._headers.append(versioned_header)
 
+        # Some values used as read-only properties so we don't recalc every time
+        self._raw = raw
+        self._offset = offset
+        self._len = len(self._raw)
+
+        # Get the payload as a byte array
+        start = self.Header.size + self.VersionHeaders[self['magic']].size
+        self._payload = self._raw[start:]
+
+        self._valid = self['checksum'] == crc32(self.payload)
+
+
     __repr__ = attribute_repr('raw', 'offset')
 
     def __len__(self):
-        return len(self.raw)
+        return self._len
 
     def __str__(self):
         return str(self.payload)
@@ -147,18 +157,25 @@ class Message(object):
         except AttributeError:
             return default
 
+    @property
+    def offset(self):
+        return self._offset
+
     @property
     def next_offset(self):
-        return self.offset + len(self)
+        return self._offset + self._len
 
     @property
     def payload(self):
-        start = self.Header.size + self.VersionHeaders[self['magic']].size
-        return self.raw[start:]
+        return self._payload
+
+    @property
+    def raw(self):
+        return self._raw
 
     @property
     def valid(self):
-        return self['checksum'] == crc32(self.payload)
+        return self._valid
 
     @classmethod
     def pack_into(cls, bytea, offset, payload, version, compression=None):
@@ -277,7 +294,8 @@ def decode_messages(payload, from_offset):
     """
     offset = 0
     recovering = False # recovering from bad offset error
-    while offset < len(payload):
+    payload_len = len(payload) # don't recalc all the time
+    while offset < payload_len:
         message = None
         try:
             header = Message.Header.unpack_from(payload, offset)
@@ -300,7 +318,12 @@ def decode_messages(payload, from_offset):
                 exception = ex
                 logger.warning('Invalid version or corrupted offset found. '
                                'Attempting recovery.')
+                logger.info('from_offset: %d\toffset: %d', from_offset, offset)
+                logger.info('payload: %s', payload)
         if message and message.valid:
+            if recovering:
+                logger.info('successfully recovered at: (%d + %d)', from_offset, offset)
+                logger.info('recovered message length: %d', length)
             recovering = False
             yield message
         elif recovering:
@@ -308,7 +331,12 @@ def decode_messages(payload, from_offset):
             offset += 1
             continue
         elif message is not None:
-            if len(message) + offset == len(payload):
+            if length > len(message):
+                if length > len(payload):
+                    raise MessageTooLargeError(
+                        'Message len %d is larger than payload len (%d)' % (
+                        length, len(payload))
+                    )
                 # If this is the last message,
                 # it's OK to drop it if it's truncated.
                 logger.debug('Discarding partial message '
@@ -513,9 +541,13 @@ class Client(object):
         response = self.handler.request(request)
 
         try:
-            return decode_messages(response.get(), from_offset=offset)
+            # N.B. Using generator here makes dealing with decode errors hard
+            return list(decode_messages(response.get(), from_offset=offset))
         except SocketDisconnectedError:
             return []
+        except MessageTooLargeError:
+            # Try again, but larger!
+            return self.fetch(topic, partition, offset, size*1.5)
 
     def multifetch(self, data):
         """
